<?php

namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class MalhaAereaController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "malhaaereadata";

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $entityManager, $newCode) {
        $this->crudService                 = $crudService;
        $this->parameterBag                = $parameterBag;
        $this->environment                 = $environment;
        $this->dataSourceManager           = $dataSourceManager;
        $this->entityManager               = $entityManager;
        $this->newCode                     = $newCode;
    }

    public function saveDatasVoo(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $row = $request->getRow();
                if($row['__new_voo']){
                   $this->dataSourceName = "malhaaereavoo";
                   parent::save($request, $response);
                }
                //if success save voo
                $this->dataSourceName = "malhaaereadata";
                $this->saveMalhaData($request, $response);
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    private function saveMalhaData(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $datasVoo = $this->_datasVoos($row, $this->dataSourceName);
        if(empty($datasVoo)){
            parent::save($request, $response);
        } else if ($row['__is_new']){
            foreach ($datasVoo as $registro){
                if(!($this->returnStrToTime($row['DTINIVIGENCIA']) > $this->returnStrToTime($registro["DTFIMVIGENCIA"]) ||
                     $this->returnStrToTime($row['DTFIMVIGENCIA']) < $this->returnStrToTime($registro["DTINIVIGENCIA"]))){
                    return $response->setError(new DTO\Response\Error('Intervalo de vigência para o voo já cadastrado!', 1));
                }
            }
            parent::save($request, $response);
        }else{
            parent::save($request, $response);          
        }
    }
    public function removeDatasVoo(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                $row = $request->getRow();
                $this->_registrosFilhos($row, "malhaaereaclasse");
                $this->dataSourceName = "malhaaereadata";
                $rows = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
                $dataSet = new DataSet($this->dataSourceName, $rows);
                $this->crudService->delete($response, $dataSet);
                //if success delete data
                if($row['__remove_voo']){
                    $this->dataSourceName = "malhaaereavoo";
                    $rows = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
                    $dataSet = new DataSet($this->dataSourceName, $rows);
                    $this->crudService->delete($response, $dataSet);
                }
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function returnStrToTime($data){
        return strtotime(str_replace('/', '-', $data));
    }

    private function _registrosFilhos($row, $dataSourceName){
        $datasVooClasse = $this->_datasVoos($row, $dataSourceName);
        if(!empty($datasVooClasse))
            throw new \Exception ("Impossível excluir. Existem classes cadastradas para a data do voo!", 1);
    }


    private function _conditionsMalhaAerea($row){
        return array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrgTrab()
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row["CDFILIAL"]
            ),
            array(
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => $row["CDCIA"]
            ),
            array(
                "columnName" => "NRMALHAAEREA",
                "operator" => "=",
                "value" => $row["NRMALHAAEREA"]
            ),
            array(
                "columnName" => "NRVOO",
                "operator" => "=",
                "value" => $row["NRVOO"]
            ),
            array(
                "columnName" => "CDTRECHO",
                "operator" => "=",
                "value" => $row["CDTRECHO"]
            )
        );
    }

    private function _datasVoos($row, $dataSourceName){
        $condition = $this->_conditionsMalhaAerea($row);
        $filterCriteria= new FilterCriteria($dataSourceName, $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $datasVoo = $dataSet->getRows();
    }
}