<?php

namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class PrevisaoConsumoController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "";
    
    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $MessagesProcedureInflightor, $GerarPrevisaoConsumoProcedure, $PrevisaoConsumoService, $entityManager) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->GerarPrevisaoConsumoProcedure = $GerarPrevisaoConsumoProcedure;
        $this->PrevisaoConsumoService = $PrevisaoConsumoService;
        $this->entityManager = $entityManager;
    }

    public function geraPrevisaoConsumo(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $row['P_CDOPERADOR'] = $this->environment->getCdOperador();
            $row['P_NRORG'] = $this->environment->getNrOrgTrab();
            $resultado = $this->GerarPrevisaoConsumoProcedure->executeProcedure($row['P_CDFILIAL'], $row['P_DATAINI'], $row['P_DATAFIN'], $row['P_CDOPERADOR'], $row['P_NRORG']);
            if($resultado['P_NRLISTAERROS'] != null) {
                $params = array(
                    'CDFILIAL'     => $row['P_CDFILIAL'],
                    'NRLISTAERROS' => $resultado['P_NRLISTAERROS']
                );
                $this->MessagesProcedureInflightor->getErrorMessages($row['P_CDFILIAL'], $resultado['P_NRLISTAERROS'], $response);
                $connection->rollback();
            } else {
                $connection->commit();
            } 
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }
    
    public function exportFileListError(DTO\Request $request, DTO\Response $response){
        $this->MessagesProcedureInflightor->concatErrorMessages($_REQUEST['cdfilial'], $_REQUEST['nrlistaerros'], $response, /*Header file*/ null, false, /*Download file error*/ true);
    }

    public function deletePrevisaoConsumo(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                    $params = array(
                        'CDFILIAL'        => $row['CDFILIAL'],
                        'NRPREVCONSINF' => $row['NRPREVCONSINF']
                    );
                    $insumos = $this->PrevisaoConsumoService->find($params, 'SELECT_INSUMOPREVCONSINF');
                    if(!empty($insumos)){
                        foreach ($insumos as $insumo) {
                            $paramsDestino = array(
                                'CDFILIAL' => $row['CDFILIAL'],
                                'NRINSUMOPREVCONSINF' => $insumo['NRINSUMOPREVCONSINF']
                            );
                            $this->PrevisaoConsumoService->delete($paramsDestino, 'DELETE_DESTPREVCONSINF');
                        }
                        $this->PrevisaoConsumoService->delete($params, 'DELETE_INSUMOPREVCONSINF');
                    }
                    $this->PrevisaoConsumoService->delete($params, 'DELETE_PREVCONSINF');
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }

}