<?php

namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class PrevisaoProducaoController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "prevpaxprodm";
    const DATASOURCE_CHECKLIST = "pcp00013_checklists_gerados";
    protected $dataSourceNameToSave = "";

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $MessagesProcedureInflightor, $entityManager) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->entityManager = $entityManager;
    }

    private function getChecklistsGerados($row, $dtprevisaoprod) {
        $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array (
                "columnName" => "DTPREVISAOPROD",
                "operator" => "=",
                "value" => substr($dtprevisaoprod, 0, 10)
            ),
            array (
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => $row['CDCIA']
            ),
            array (
                "columnName" => "NRVOO",
                "operator" => "=",
                "value" => "T"
            ),
            array (
                "columnName" => "CDTRECHO",
                "operator" => "=",
                "value" => "T"
            ),
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            ),
        );
        $dataSet = $this->crudService->getFilteredDataSet(self::DATASOURCE_CHECKLIST, $conditions);
        if(empty($dataSet->getRows()))
            return [];
        else 
            return $dataSet->getRows();
    }

    private function getVoos($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $row['NRPREVISAOPROD']
            ),
            array(
                'columnName' => 'CDCIA',
                'operator' => '=',
                'value' => $row['CDCIA']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prevpaxvooprodm', $conditions);
        return $dataSet->getRows();
    }
    private function getCompanhias($request){
        $rows = $request->getDataSet()->getRows();
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $rows[0]['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $rows[0]['NRPREVISAOPROD']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prevpaxciaprodm', $conditions);
        return $dataSet->getRows();
    }
    private function getStatusCompanhias($row){
        $status = 'A';
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $row['NRPREVISAOPROD']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prevpaxciaprodm', $conditions);
        foreach ($dataSet->getRows() as $companhias) {
            if($companhias['IDPREVISAOPROD'] != 'A' && $companhias['IDPREVISAOPROD'] != 'C')
                $status = $companhias['IDPREVISAOPROD'];
        }
        return $status;
    }
    private function getPrevisao($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $row['NRPREVISAOPROD']
            ),
            array(
                'columnName' => 'DTPREVISAOPROD',
                'operator' => '=',
                'value' => $row['DTPREVISAOPROD']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceName, $conditions);
        return $dataSet->getRows();
    }

    private function getPrevisaoClasses($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $row['NRPREVISAOPROD']
            ),
            array(
                'columnName' => 'CDCIA',
                'operator' => '=',
                'value' => $row['CDCIA']                
            ),
            array(
                'columnName' => 'NRVOO',
                'operator' => '=',
                'value' => $row['NRVOO']                
            ),
            array(
                'columnName' => 'CDTRECHO',
                'operator' => '=',
                'value' => $row['CDTRECHO']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prevpaxclaprodm', $conditions);
        return $dataSet->getRows();
    }
    
    private function getStatusVoos($row){
        $status = 'A';
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRPREVISAOPROD',
                'operator' => '=',
                'value' => $row['NRPREVISAOPROD']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prevpaxvooprodm', $conditions);
        foreach ($dataSet->getRows() as $voos) {
            if($voos['IDPREVISAOPROD'] != 'A' && $voos['IDPREVISAOPROD'] != 'C')
                $status = $voos['IDPREVISAOPROD'];
        }
        return $status;
    }
    private function _validateChecklists(DTO\Request\DataSet $request, DTO\Response $response){
        $checklistsMessages = array();
        array_push($checklistsMessages, "PCP00013_PREV_CHECKLIST_GER");
        $rows = $request->getDataSet()->getRows();
        $dtprevisaoprod = $rows[0]['DTPREVISAOPROD'];
        foreach ($rows as $companhia) {
            $checklistsCompanhia = $this->getChecklistsGerados($companhia, $dtprevisaoprod);
            if(!empty($checklistsCompanhia)){
                $checklistsDistinct = $this->unique_multidim_array($checklistsCompanhia, "NRCKLISTPRODINF");
                foreach ($checklistsDistinct as $checklist) {
                    array_push($checklistsMessages, "PCP00013_PREV_CHECKLIST_GER_VOO" . "#_#". $companhia['VOOCIA'] . "#_#". $companhia['TRECHO'] . "#_#". $checklist['NRCKLISTPRODINF']);
                }
            }
        }
        if(count($checklistsMessages) == 1){
            return  [];
        } else {
            return $checklistsMessages;
        }
    }

    public function aprovarCompanhia(DTO\Request\DataSet $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $this->dataSourceNameToSave = "prevpaxciaprodm";
        try {
            $connection->beginTransaction();
            if($this->_validateChecklists($request, $response)){
                $STATUS = $request->getDataSet()->getRows()[0]['STATUS'];
                foreach ($request->getDataSet()->getRows() as $row) {
                    //Aprova todos os voos da companhia
                    $voos = $this->getVoos($row);
                    //$this->_validateQtdClasse($voos);
                    $previsoesErrorMsg = $this->_aprovaVoos($voos, 'cia', $request, $STATUS);
                    $row['IDPREVISAOPROD'] = $STATUS;
                }
                //Aprova a Companhia
                $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
                $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rows);
                //Verifica se todas as Companhias que decolam no dia estão aprovadas
                if($this->getStatusCompanhias($request->getDataSet()->getRows()[0]) == $STATUS){
                    //Aprova o dia
                    $this->aprovaPrevisao($request, $STATUS);
                    $this->addDataset($response, $cia = true, $day = true);
                } else {
                    //Ainda existem companhias pendentes na Previsão
                    $this->addDataset($response, $cia = false, $day = false);
                }
            }
            //commit
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    public function aprovarVoo(DTO\Request\DataSet $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $this->dataSourceNameToSave = "prevpaxciaprodm";
        try {
            $connection->beginTransaction();
            //Aprova os voos da companhia
            //$voos = $this->getVoos($row);
            $checklistsGerados = $this->_validateChecklists($request, $response);
                $STATUS = $request->getDataSet()->getRows()[0]['STATUS'];
                $previsoesErrorMsg = $this->_aprovaVoos(false, 'voo', $request, $STATUS);
                $errorMsgs = array_merge($checklistsGerados, $this->_concatErrorMessages($STATUS, $previsoesErrorMsg));
                $statusVoos = $this->getStatusVoos($request->getDataSet()->getRows()[0]);
                if($STATUS == 'C' && $statusVoos == 'A'){
                    $STATUS = 'A';
                }
                if($statusVoos == $STATUS){
                    //Aprova a companhia quando todos os Voos estão aprovados
                    $this->aprovaCompanhia($request, $STATUS);
                    //Verifica se todas as Companhias que decolam no dia estão aprovadas
                    if($this->getStatusCompanhias($request->getDataSet()->getRows()[0]) == $STATUS){
                        //Aprova o dia
                        $this->aprovaPrevisao($request, $STATUS);
                        $this->addDataset($response, $cia = true, $day = true, $errorMsgs);
                    } else {
                        //Ainda existem companhias pendentes na Previsão
                        $this->addDataset($response, $cia = true, $day = false, $errorMsgs);
                    }
                } else {
                    //Ainda existem Voos pendentes na Previsão
                    $this->addDataset($response, $cia = false, $day = false, $errorMsgs);
                }
            //commit
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    private function _aprovaVoos($voos, $origin, $request, $STATUS){
        $rowsSave = array();
        if(strcmp($origin, 'cia') != 0){
            foreach ($request->getDataSet()->getRows() as &$voo){
                $voo['IDPREVISAOPROD'] = $STATUS;
                $voo['__is_new'] = false;
                $voo['changed'] = true;
            }
            $rows = $this->crudService->getRowsToSave($request, 'prevpaxvooprodm');
            $rowsSave = $rows;
        } else {
            foreach ($voos as &$voo){
                $voo['IDPREVISAOPROD'] = $STATUS;
                $voo['__is_new'] = false;
                $voo['changed'] = true;
            }
            $rowsSave = $voos;
        }
        $previsoesError = $this->_beforeAprovaVoo($STATUS, $rowsSave);
        $permiteSave = $this->_filterRowsSaveByErros($previsoesError, $rowsSave);
        $this->crudService->save('prevpaxvooprodm', $permiteSave);
        return $previsoesError;
    }

    private function _filterRowsSaveByErros($erros, $rows){
        if(empty($erros)){
            return $rows;
        } else {
            foreach ($erros as $vooInvalido) {
                $filtro = array(
                    "CDFILIAL" => $vooInvalido['CDFILIAL'],
                    "NRPREVISAOPROD" => $vooInvalido['NRPREVISAOPROD'],
                    "CDCIA" => $vooInvalido['CDCIA'],
                    "NRVOO" => $vooInvalido['NRVOO'],
                    "CDTRECHO" => $vooInvalido['CDTRECHO']
                 );
                 foreach($rows as $key => $row){
                    $count = 0; 
                    foreach($filtro as $indx => $value){
                         if($row[$indx] == $value){
                             $count++;
                         }
                     }
                     if($count == count($filtro)){
                         unset($rows[$key]);
                     }
                 }
            }
            return $rows;
        }
    }

    private function  _beforeAprovaVoo($status, $rows){
        $previsoesSemClasse = array();
        if($status == 'A'){
            foreach ($rows as $voo) {
                $vooClasse = array();
                $vooClasse = $this->getPrevisaoClasses($voo);
                if(empty($vooClasse)){
                    array_push($previsoesSemClasse, $voo);
                }
            }
        }
        return $previsoesSemClasse;
    }

    private function  _concatErrorMessages($status, $rows){
        $previsoesSemClasseMsgs = array();
        array_push($previsoesSemClasseMsgs, "PCP00013_PREV_SEM_CLASSE");
        if($status == 'A'){
            foreach ($rows as $voo) {
                array_push($previsoesSemClasseMsgs, "PCP00013_PREV_SEM_CLASSE_VOO" . "#_#" . $voo['VOOCIA']. "#_#" .$voo['TRECHO']. "#_#" .$voo['NRPREVISAOPROD']);
            }
        }
        if(count($previsoesSemClasseMsgs) == 1){
            return  [];
        } else {
            return $previsoesSemClasseMsgs;
        }
    }

    private function aprovaCompanhia($request, $STATUS){
        try {
            $cias = $this->getCompanhias($request);
            foreach ($cias as &$companhia){
                $companhia['IDPREVISAOPROD'] = $STATUS;
                $companhia['__is_new'] = false;
                $companhia['changed'] = true;
            }
            $newDataset = new DataSet('prevpaxciaprodm', $cias);
            $newRequest = new DTO\Request\DataSet($newDataset, $request->getMethod(), $request->getRoutePath(), $request->getUserId());   
            $rows = $this->crudService->getRowsToSave($newRequest, 'prevpaxciaprodm');
            $rowsSave = $this->unique_multidim_array($rows, "CDCIA");
            $this->crudService->save('prevpaxciaprodm', $rowsSave);
        } catch (\Exception $e){
            throw new \Exception($e->getMessage(), 1);
        }
    }
    private function aprovaPrevisao($request, $STATUS){
        try {
            $previsao = $this->getPrevisao($request->getDataSet()->getRows()[0]);
            foreach ($previsao as &$dia){
                $dia['IDPREVISAOPROD'] = $STATUS;
                $dia['__is_new'] = false;
            }
            $this->crudService->save($this->dataSourceName, $previsao);
        } catch (\Exception $e){
            throw new \Exception($e->getMessage(), 1);
        }
    }
    private function addDataset($response, $cia, $day, $msgerror){
        $response->addDataset(new DataSet('approvedPrevisao',
            array(
                'approved_cia' => $cia,
                'approved_day' => $day,
                'msgerror'     => $msgerror
            )));
    }
    private function unique_multidim_array($array, $key) { 
        $temp_array = array(); 
        $i = 0; 
        $key_array = array(); 
        
        foreach($array as $val) { 
            if (!in_array($val[$key], $key_array)) { 
                $key_array[$i] = $val[$key]; 
                $temp_array[$i] = $val; 
            } 
            $i++; 
        } 
        return $temp_array; 
    }
}