<?php
namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Report\Strategy\BirtStrategy;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class RelatorioEstimativaRefeicoes extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $birtStrategy;
    protected $connection;
    protected $crudService;

    public function __construct(ReportService $reportService, $environment, $connection, $birtStrategy, $crudService) {
        $this->environment = $environment;
        $this->connection = $connection;
        $this->birtStrategy = $birtStrategy;
        $this->crudService = $crudService;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'PCP00400' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "CDCIA"            => "CDCIA",
              "CIAVOOTRECHO"     => "NRVOO",
              "DATAINI"          => "DATAINI",
              "DATAFIN"          => "DATAFIN",
              "NACIONALIDADE"    => "NACIONALIDADE",
              "NRORG"            => "NRORG",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00401' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "CDCIA"            => "CDCIA",
              "CIAVOOTRECHO"     => "NRVOO",
              "DATAINI"          => "DATAINI",
              "DATAFIN"          => "DATAFIN",
              "NACIONALIDADE"    => "NACIONALIDADE",
              "NRORG"            => "NRORG",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00403' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"            => "CDFILIAL",
              "CDCIA"               => "CDCIA",
              "CIAVOOTRECHO"        => "NRVOO",
              "NACIONALIDADE"       => "NACIONALIDADE",
              "DTMESANO"            => "DTMESANO",
              "NRORG"               => "NRORG",
              "REPORTFORMAT"        => "__format"
            )
        ),
        'PCP00404' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "CDFILIAL"            => "CDFILIAL",
                "NACIONALIDADE"       => "NACIONALIDADE",
                "CDCIA"               => "CDCIA",
                "CIASERVICO"          => "NRSERVICOCIAFILI",
                "CIAVOOTRECHO"        => "NRVOO",
                "DATAINI"             => "DATAINI",
                "DATAFIN"             => "DATAFIN",
                "NRORG"               => "NRORG",
                "REPORTFORMAT"        => "__format"
            )
        ),
        'PCP00405' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "CDFILIAL"            => "CDFILIAL",
                "NACIONALIDADE"       => "NACIONALIDADE",
                "CDCIA"               => "CDCIA",
                "CIAVOOTRECHO"               => "NRVOO",
                "CIASERVICO"    => "NRSERVICOCIAFILI",
                "DTMESANO"            => "DTMESANO",
                "NRORG"               => "NRORG",
                "REPORTFORMAT"        => "__format"
            )
        ),
        'PCP00406' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "CDFILIAL"            => "CDFILIAL",
                "NACIONALIDADE"       => "NACIONALIDADE",
                "CDCIA"               => "CDCIA",
                "CIAVOOTRECHO"               => "NRVOO",
                "DATAINI"             => "DATAINI",
                "DATAFIN"             => "DATAFIN",
                "NRORG"               => "NRORG",
                "REPORTFORMAT"        => "__format"
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat($row['REPORTFORMAT']);
        $row['NRORG'] = $this->environment->getNrOrgTrab();
    }
}