<?php
namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Report\Strategy\BirtStrategy;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class RelatorioProducaoReceita extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $birtStrategy;
    protected $connection;
    protected $crudService;   
    protected $dataSourceName = "";
    protected $dataSourceNameChecklist = "cklistprodinf";
    protected $MessagesProcedureInflightor;
    const DATASOURCE_CHECKLIST = "pcp00030_checklist_aprovados_emitidos";


    public function __construct(ReportService $reportService, $environment, $connection, $birtStrategy, $crudService, $MessagesProcedureInflightor) {
        $this->birtStrategy                = $birtStrategy;
        $this->crudService                 = $crudService;
        $this->environment                 = $environment;
        $this->connection                  = $connection;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'PCP00001' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "DTINIVIGENCIA"    => "DTINIVIGENCIA",
              "DTFIMVIGENCIA"    => "DTFIMVIGENCIA",
              "CDSETOR"          => "CDSETOR",
              "NRORG"            => "NRORG",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00002' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "DTINIVIGENCIA"    => "DTINIVIGENCIA",
              "DTFIMVIGENCIA"    => "DTFIMVIGENCIA",
              "CDSETOR"          => "CDSETOR",
              "NRORG"            => "NRORG",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00003' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "DTMESANO"         => "DTMESANO",
              "CDSETOR"          => "CDSETOR",
              "NRORG"            => "NRORG",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00004' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         => "CDFILIAL",
              "NRLISTA"          => "NRLISTA",
              "DTINIVIGENCIA"    => "DTINIVIGENCIA",
              "DTFIMVIGENCIA"    => "DTFIMVIGENCIA",
              "REPORTFORMAT"     => "__format"
            )
        ),
        'PCP00300' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"           => "CDFILIAL",
              "T_SETOR"            => "T_SETOR",
              "T_TURNO"            => "T_TURNO",
              "T_CATEGORIAPROD"    => "T_CATEGORIAPROD",
              "T_SUBCATEGORIAPROD" => "T_SUBCATEGORIAPROD",
              "T_TIPOPROD"         => "T_TIPOPROD",
              "T_TIPONEGOCIOINF"   => "T_TIPONEGOCIOINF",
              "APENASTOTAL"        => "APENASTOTAL",
              "DTREF"              => "DTREF",
              "NRORG"              => "NRORG",
              "REPORTFORMAT"       => "__format",
              "PREENCHEMANUAL"     => "PREENCHEMANUAL"
            )
        ),
        'PCP00301' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"           => "CDFILIAL",
              "T_SETOR"            => "T_SETOR",
              "T_TURNO"            => "T_TURNO",
              "T_CATEGORIAPROD"    => "T_CATEGORIAPROD",
              "T_SUBCATEGORIAPROD" => "T_SUBCATEGORIAPROD",
              "T_TIPOPROD"         => "T_TIPOPROD",
              "T_TIPONEGOCIOINF"   => "T_TIPONEGOCIOINF",
              "APENASTOTAL"        => "APENASTOTAL",
              "DTREF"              => "DTREF",
              "NRORG"              => "NRORG",
              "REPORTFORMAT"       => "__format",
              "PREENCHEMANUAL"     => "PREENCHEMANUAL"
            )
        ),
        'PCP00302' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"           => "CDFILIAL",
              "T_SETOR"            => "T_SETOR",
              "T_TURNO"            => "T_TURNO",
              "T_CATEGORIAPROD"    => "T_CATEGORIAPROD",
              "T_SUBCATEGORIAPROD" => "T_SUBCATEGORIAPROD",
              "T_TIPOPROD"         => "T_TIPOPROD",
              "T_TIPONEGOCIOINF"   => "T_TIPONEGOCIOINF",
              "APENASTOTAL"        => "APENASTOTAL",
              "DTREF"              => "DTREF",
              "NRORG"              => "NRORG",
              "REPORTFORMAT"       => "__format",
              "PREENCHEMANUAL"     => "PREENCHEMANUAL"
            )
        ),
        'PCP00310' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"           => "CDFILIAL",
              "T_SETOR"            => "T_SETOR",
              "T_TURNO"            => "T_TURNO",
              "T_CATEGORIAPROD"    => "T_CATEGORIAPROD",
              "T_SUBCATEGORIAPROD" => "T_SUBCATEGORIAPROD",
              "T_TIPOPROD"         => "T_TIPOPROD",
              "T_TIPONEGOCIOINF"   => "T_TIPONEGOCIOINF",
              "T_RECEITA"          => "T_RECEITA",
              "DTREFINI"           => "DTREFINI",
              "DTREFFIM"           => "DTREFFIM",
              "NRORG"              => "NRORG",
              "REPORTFORMAT"       => "__format"
            )
        ),
        'PCP00311' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"           => "CDFILIAL",
              "T_SETOR"            => "T_SETOR",
              "T_TURNO"            => "T_TURNO",
              "T_CATEGORIAPROD"    => "T_CATEGORIAPROD",
              "T_SUBCATEGORIAPROD" => "T_SUBCATEGORIAPROD",
              "T_TIPOPROD"         => "T_TIPOPROD",
              "T_TIPONEGOCIOINF"   => "T_TIPONEGOCIOINF",
              "T_RECEITA"          => "T_RECEITA",
              "DTREFINI"           => "DTREFINI",
              "DTREFFIM"           => "DTREFFIM",
              "NRORG"              => "NRORG",
              "REPORTFORMAT"       => "__format"
            )
        ),
        'PCP00312' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"     => "CDFILIAL",
              "DTREFINI"     => "DTINICIAL",
              "DTREFFIM"     => "DTFINAL",
              "T_SETOR"      => "CDSETOR",
              "T_TURNO"      => "NRTURNO",
              "T_TIPOPROD"   => "NRTIPOPROD",
              "NRORG"        => "NRORG",
              "REPORTFORMAT" => "__format"
            )
        ),
        'PCP00330' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"     => "CDFILIAL",
              "T_SETOR"      => "T_SETOR",
              "T_TURNO"      => "T_TURNO",
              "T_REQUISICAO" => "T_REQUISICAO",
              "T_PRODUTO"    => "T_PRODUTO",
              "DTREFINI"     => "DTREFINI",
              "DTREFFIM"     => "DTREFFIM",
              "NRORG"        => "NRORG",
              "REPORTFORMAT" => "__format"
            )
        ),
        'PCP00331' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"     => "CDFILIAL",
              "T_SETOR"      => "T_SETOR",
              "T_TURNO"      => "T_TURNO",
              "T_RETIRADA"   => "T_RETIRADA",
              "T_PRODUTO"    => "T_PRODUTO",
              "DTREFINI"     => "DTREFINI",
              "DTREFFIM"     => "DTREFFIM",
              "NRORG"        => "NRORG",
              "REPORTFORMAT" => "__format"
            )
        ),
        'PCP00332' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"     => "CDFILIAL",
              "T_SETOR"      => "T_SETOR",
              "T_TURNO"      => "T_TURNO",
              "T_REQUISICAO" => "T_REQUISICAO",
              "T_PRODUTO"    => "T_PRODUTO",
              "DTREFINI"     => "DTREFINI",
              "DTREFFIM"     => "DTREFFIM",
              "NRORG"        => "NRORG",
              "REPORTFORMAT" => "__format"
            )
        ),
        'PCP00333' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"     => "CDFILIAL",
              "T_SETOR"      => "T_SETOR",
              "T_TURNO"      => "T_TURNO",
              "T_RETIRADA"   => "T_RETIRADA",
              "T_PRODUTO"    => "T_PRODUTO",
              "DTREFINI"     => "DTREFINI",
              "DTREFFIM"     => "DTREFFIM",
              "NRORG"        => "NRORG",
              "REPORTFORMAT" => "__format"
            )
        )
    );

  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
      $row = $request->getRow();
      $this->birtStrategy->setReportFormat($row['REPORTFORMAT']);
      $row['NRORG'] = $this->environment->getNrOrgTrab();
      if($row["REPORTNAME"] === "PCP00004"){
          $this->executaProcedureRetornaNrlista($row, $response);
      }
  }
  
  public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report){
      $row = $request->getRow();
      //Altera status do Checklist para Emitido
      if($row['REPORTNAME'] === "PCP00300"){
        $checklists = $this->findChecklists($row);
        foreach ($checklists as &$cklist) {
            $cklist['__is_new'] = false;
            $cklist['changed'] = false;
            $cklist['IDSITUACAO'] = "EMITIDO";
        }
        $this->effectSave($checklists, $response);
      }
  }
  
  private function findChecklists($row){
      $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array (
                "columnName" => "T_SETOR",
                "operator" => "=",
                "value" => $row['T_SETOR']
            ),
            array (
                "columnName" => "T_TURNO",
                "operator" => "=",
                "value" => $row['T_TURNO']
            ),
            array (
                "columnName" => "T_CATEGORIA",
                "operator" => "=",
                "value" => $row['T_CATEGORIAPROD']
            ),
            array (
                "columnName" => "T_SUBCATEGORIA",
                "operator" => "=",
                "value" => $row['T_SUBCATEGORIAPROD']
            ),
            array (
                "columnName" => "T_TIPOPROD",
                "operator" => "=",
                "value" => $row['T_TIPOPROD']
            ),
            array (
                "columnName" => "T_TIPONEGOCIOINF",
                "operator" => "=",
                "value" => $row['T_TIPONEGOCIOINF']
            ),
            array (
                "columnName" => "DTREF",
                "operator" => "=",
                "value" => $row['DTREF']
            ),         
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            ),
        );
        
        $dataSet = $this->crudService->getFilteredDataSet(self::DATASOURCE_CHECKLIST, $conditions);
        if(empty($dataSet->getRows()))
            return [];
        else 
            return $dataSet->getRows();
  }
  
  private function effectSave($checklists, $response){
        $dataSet = new DataSet($this->dataSourceNameChecklist, $checklists);
        $request = new DTO\Request\DataSet($dataSet, '','','');
        $rowToSave = $this->crudService->getRowsToSave($request, $this->dataSourceNameChecklist);
        $this->crudService->save($this->dataSourceNameChecklist, $rowToSave);
  }
  
  private function executaProcedureRetornaNrlista($row, $response){
      $nrlista = $this->identificaPendparamprod($row);
      $row['NRLISTA'] = $nrlista;
  }
  
  private function identificaPendparamprod($row){
      $procedure = new StoredProcedure($this->connection, 'IDENTIFICA_PENDPARAMPROD');
      $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDCIA',        Param::PARAM_INPUT,  $row['CDCIA'],                       Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_DTINICIO',     Param::PARAM_INPUT,  $row['DTINIVIGENCIA'],               Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_DTFIM',        Param::PARAM_INPUT,  $row['DTFIMVIGENCIA'],               Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDSETOR',      Param::PARAM_INPUT,  $row['CDSETOR'],                     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT,  $row['NRORG'],                       Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, '',                                  Param::PARAM_TYPE_INT, 20));
      $resultado = $procedure->execute();
      return $resultado['P_NRLISTAERROS'];
  }
  
}
