<?php
namespace Teknisa\PCP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class SolicitacaoCheckListEmergencialController {

    protected $procedureAprovaSolicitacao;
    protected $procedureCancelaAprovaSolicitacao;
    protected $connection;
    protected $messagesProcedureInflightor;

    public function __construct($procedureAprovaSolicitacao, $procedureCancelaAprovaSolicitacao, $connection, $messagesProcedureInflightor) {
        
        $this->procedureAprovaSolicitacao           = $procedureAprovaSolicitacao;
        $this->procedureCancelaAprovaSolicitacao    = $procedureCancelaAprovaSolicitacao;
        $this->connection                           = $connection;
        $this->messagesProcedureInflightor          = $messagesProcedureInflightor;
        
    }
    
    public function callProcedureApproveSolicitation(Request $request, Response $response){
        
        $row = $request->getRow();
        
        try {
            
            $this->connection->beginTransaction();
            
            $resultado = $this->procedureAprovaSolicitacao->executeProcedure($row);
            if($resultado['P_NRLISTAERROS'] != null) {
                $this->messagesProcedureInflightor->getErrorMessages($row['CDFILIAL'], $resultado['P_NRLISTAERROS'], $response);
                $this->connection->rollback();
                return;
            }
            
            $this->connection->commit();
            
        } catch (\Exception $e){
            
            $this->connection->rollback();
            $response->setError(new Error($this->messagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            
        }
        
        
    }
    
    public function callProcedureCancelApproveSolicitation(Request $request, Response $response){
        
        $row = $request->getRow();
        
        try {
            
            $this->connection->beginTransaction();
            
            $resultado = $this->procedureCancelaAprovaSolicitacao->executeProcedure($row);
            if($resultado['P_NRLISTAERROS'] != null) {
                $this->messagesProcedureInflightor->getErrorMessages($row['CDFILIAL'], $resultado['P_NRLISTAERROS'], $response);
                $this->connection->rollback();
                return;
            }
            
            $this->connection->commit();
            
        } catch (\Exception $e){
            
            $this->connection->rollback();
            $response->setError(new Error($this->messagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
            
        }
        
    }
    
}