<?php

 /**
  * @author Fabio Pereira <fabio.pereira@teknisa.com>
  * 01/11/2017
  */

namespace Teknisa\PCP\Services;

use Zeedhi\Framework\DTO\Response\Error;
use \Doctrine\ORM\EntityManager;

class RateioService {

	/**
	 * @var EntityManager connection
	 */
	protected $connection;
	/**
	 * @var EntityManager connection
	 */
	protected $entityManager;
	/**
	 * Column used as base for calculation
	 * @var String columnOrigin
	 */
	protected $columnOrigin;
	/**
	 * Column affected by calculation
	 * @var String columnOrigin
	 */
	protected $columnAffected;
	/**
	 * Column used as the basis for destination calculation
	 * @var String columnOrigin
	 */
	protected $columnReference;

	public function __construct(EntityManager $entityManager) {
		$this->connection 		= $entityManager->getConnection();
		$this->entityManager 	= $entityManager;
		$this->columnOrigin 	= "";
		$this->columnAffected 	= "";
		$this->columnReferences = "";
	}

	/**
	 * Set $columnOrigin;
	 * @param string $origin
	*/
	public function setOriginColumns($origin){
		$this->columnOrigin = $origin;
	}
	/**
	 * Set $columnAffected;
	 * @param string $affected
	*/
	public function setAffectedColumns($affected){
		$this->columnAffected = $affected;
	}
	/**
	 * Set $columnReference;
	 * @param string $reference
	*/
	public function setReferenceColumns($reference){
		$this->columnReference = $reference;
	}
	/**
	 * Returns the sum of all values in columnReference
	 * @param array $arrayAffected
	 * @return float
	 */
	private function getSumByColumnAffected($arrayAffected){
		if(!is_array($arrayAffected)){
			$arrayAffected = array($arrayAffected);
		}
		return array_reduce($arrayAffected, function($i, $obj){
		    return $i += $obj[$this->columnReference];
		});
	}
	/**
	 * Fixes rounding error in percentage calculation
	 * @param float $totalCalc
	 * @param float $newTotalAmount
	 * @param array $arrayAffected
	 * @return array
	 */
	private function fixesErrorByRound($totalCalc, $newTotalAmount, $arrayAffected){
		$diif = round($newTotalAmount - $totalCalc, 3);
		if($diif !== 0.000){
			$maxValue = $arrayAffected[0][$this->columnAffected];
			$maxKey = 0;
			foreach ($arrayAffected as $key => $row) {
				if($maxValue < $row[$this->columnAffected]){
					$maxKey = $key;
					$maxValue = $row[$this->columnAffected];
				}
			}
			$arrayAffected[$maxKey][$this->columnAffected] = $arrayAffected[$maxKey][$this->columnAffected] + $diif;
		}
		return $arrayAffected;
	}
	/**
	 *Calculates the RATEIO based on the parameterized columns
	 * @param row $rowOrigin
	 * @param array $arrayAffected
	 * @return array arrayRateio new array based in the param arrayAffected
	 */
	public function calcRateio($rowOrigin, $arrayAffected){
		if(!empty($this->columnAffected) && !empty($this->columnOrigin)){
			if(!empty($rowOrigin) && !empty($arrayAffected)){
				if(empty($this->columnReference)){
					$this->setReferenceColumns($this->columnAffected);
				}
				$arrayRateio = array();
				$newTotalAmount = $rowOrigin[$this->columnOrigin];
				$oldTotalAmount = $this->getSumByColumnAffected($arrayAffected);
				if($newTotalAmount !== $oldTotalAmount){
					$totalCalc = 0.000;
					$oldTotalAmount = $oldTotalAmount == 0 ? $newTotalAmount : $oldTotalAmount;
					foreach ($arrayAffected as &$rowAffected) {
						$percent = (($rowAffected[$this->columnReference] * 100)/$oldTotalAmount)/100;
						$rowAffected[$this->columnAffected] = round($percent * $newTotalAmount, 3);
						$totalCalc += $rowAffected[$this->columnAffected];
					}
					$arrayRateio = $this->fixesErrorByRound($totalCalc, $newTotalAmount, $arrayAffected);
				}
				return $arrayRateio;
			} else {
				throw new \Exception("Rows not defined!", 1);
			}
		} else {
			throw new \Exception("Columns not defined!", 1);
		}
	}

}